
/**************************************************************************
 *
 * Copyright 2016 Samsung Electronics co., Ltd. All Rights Reserved.
 *
 * Contact: Konstantin Drabeniuk <k.drabeniuk@samsung.com>
 * Contact: Andrii Sokolenko <a.sokolenko@samsung.com>
 * Contact: Roman Marchenko <r.marchenko@samsung.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
**************************************************************************/

#include "tc_tbm.h"

class TBMBo : public TBMBufmgr
{
public:
	TBMBo();
	void SetUp(void);
	void TearDown(void);

	unsigned int bufmgr_capability;
};

TBMBo::TBMBo()
{
	bufmgr_capability = TBM_BUFMGR_CAPABILITY_NONE;
}

void TBMBo::SetUp()
{
	TBMBufmgr::SetUp();

	bufmgr_capability = tbm_bufmgr_get_capability(bufmgr);
	EXPECT_TRUE(bufmgr_capability != TBM_BUFMGR_CAPABILITY_NONE);
}

void TBMBo::TearDown()
{
	TBMBufmgr::TearDown();
}

static int memory_for_invalid_param;
static tbm_bufmgr invalid_bufmgr = (tbm_bufmgr)&memory_for_invalid_param;
static tbm_bo invalid_bo = (tbm_bo)&memory_for_invalid_param;
static tbm_bo invalid_bo2 = (tbm_bo)&memory_for_invalid_param;

/* tbm_bo_alloc() */

TEST_F(TBMBo, BoAllocUnref)
{
	tbm_bo bo;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
}

TEST_F(TBMBo, BoAllocWithBufmgrParamTest)
{
	tbm_bo bo;

	bo = tbm_bo_alloc(NULL, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo = tbm_bo_alloc(invalid_bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoAllocWithSizeParamTest)
{
	tbm_bo bo;

	bo = tbm_bo_alloc(bufmgr, -1, TBM_BO_DEFAULT);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoAllocWithFlagsParamTest)
{
	tbm_bo bo;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	tbm_bo_unref(bo);

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_SCANOUT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	tbm_bo_unref(bo);

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_NONCACHABLE);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	tbm_bo_unref(bo);

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_WC);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	tbm_bo_unref(bo);
}

/* tbm_bo_ref() */

TEST_F(TBMBo, BoRef)
{
	tbm_bo bo;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo = tbm_bo_ref(bo);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	tbm_bo_unref(bo);
}

TEST_F(TBMBo, BoRefWithBoParamTest)
{
	tbm_bo bo;

	bo = tbm_bo_ref(NULL);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo = tbm_bo_ref(invalid_bo);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_bo_unref() */

TEST_F(TBMBo, BoUnrefWithBoParamTest)
{
	tbm_bo_unref(NULL);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bo_unref(invalid_bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_bo_map() */

TEST_F(TBMBo, BoMapUnmap)
{
	tbm_bo bo;
	tbm_bo_handle bo_handle;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
}

TEST_F(TBMBo, BoMapWithBoParamTest)
{
	tbm_bo_handle bo_handle;

	bo_handle = tbm_bo_map(NULL, TBM_DEVICE_CPU, TBM_OPTION_READ);
	EXPECT_EQ(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo_handle = tbm_bo_map(invalid_bo, TBM_DEVICE_CPU, TBM_OPTION_READ);
	EXPECT_EQ(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

// TODO:: Use Value-Parameterized??
TEST_F(TBMBo, BoMapWithDeviceParamTest)
{
	tbm_bo bo;
	tbm_bo_handle bo_handle;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_DEFAULT, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_2D, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_3D, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

#if 0
	bo_handle = tbm_bo_map(bo, TBM_DEVICE_MM, TBM_OPTION_READ);
	if (!bo_handle.ptr)
	EXPECT_NE(bo_handle.ptr, nullptr);
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
#endif

	tbm_bo_unref(bo);
}

// TODO:: Use Value-Parameterized??
TEST_F(TBMBo, BoMapWithOptParamTest)
{
	tbm_bo bo;
	tbm_bo_handle bo_handle;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_WRITE);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ|TBM_OPTION_WRITE);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
}

/* tbm_bo_unmap() */

TEST_F(TBMBo, BoUnmapWithBoParamTest)
{
	tbm_bo bo;
	int ret;

	ret = tbm_bo_unmap(NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_unmap(invalid_bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_unmap(invalid_bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bo_unref(bo);
}

/* tbm_bo_get_handle() */

TEST_F(TBMBo, BoGetHandle)
{
	tbm_bo bo;
	tbm_bo_handle bo_handle;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	bo_handle = tbm_bo_get_handle(bo, TBM_DEVICE_CPU);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	tbm_bo_unref(bo);
}

TEST_F(TBMBo, BoGetHandleWithBoParamTest)
{
	tbm_bo_handle bo_handle;
	bo_handle = tbm_bo_get_handle(NULL, TBM_DEVICE_DEFAULT);
	EXPECT_EQ(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo_handle = tbm_bo_get_handle(invalid_bo, TBM_DEVICE_DEFAULT);
	EXPECT_EQ(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoGetHandleWithDeviceParamTest)
{
	tbm_bo bo;
	tbm_bo_handle bo_handle;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_get_handle(bo, TBM_DEVICE_DEFAULT);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_get_handle(bo, TBM_DEVICE_CPU);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_get_handle(bo, TBM_DEVICE_2D);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_get_handle(bo, TBM_DEVICE_3D);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

#if 0
	bo_handle = tbm_bo_get_handle(bo, TBM_DEVICE_MM);
	EXPECT_NE(bo_handle.ptr, nullptr);
#endif

	tbm_bo_unref(bo);
}

/* tbm_bo_export() */

TEST_F(TBMBo, BoExport)
{
	tbm_bo bo;
	tbm_key bo_key;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_KEY))
		return;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_key = tbm_bo_export(bo);
	EXPECT_GT(bo_key, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoExportWithBoParamTest)
{
	tbm_key bo_key;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_KEY))
		return;

	bo_key = tbm_bo_export(NULL);
	EXPECT_EQ(bo_key, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo_key = tbm_bo_export(invalid_bo);
	EXPECT_EQ(bo_key, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_bo_export_fd() */

TEST_F(TBMBo, BoExportFd)
{
	tbm_bo bo;
	tbm_fd bo_fd;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_FD))
		return;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_fd = tbm_bo_export_fd(bo);
	EXPECT_GT(bo_fd, -1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoExportFdWithBoParamTest)
{
	tbm_fd bo_fd;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_FD))
		return;

	bo_fd = tbm_bo_export_fd(NULL);
	EXPECT_EQ(bo_fd, -1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo_fd = tbm_bo_export_fd(invalid_bo);
	EXPECT_EQ(bo_fd, -1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_bo_import() */

TEST_F(TBMBo, BoImport)
{
	tbm_bo bo1, bo2;
	tbm_key bo_key;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_KEY))
		return;

	bo1 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo1, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_key = tbm_bo_export(bo1);
	EXPECT_GT(bo_key, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo2 = tbm_bo_import(bufmgr, bo_key);
	EXPECT_NE(bo2, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoImportWithBufmgrParamTest)
{
	tbm_bo bo;
	tbm_key bo_key = (tbm_key)TBM_UT_INVALID_UINT_VALUE;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_KEY))
		return;

	bo = tbm_bo_import(NULL, bo_key);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo = tbm_bo_import(invalid_bufmgr, bo_key);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoImportWithKeyParamTest)
{
	tbm_bo bo;
	tbm_key bo_key_invalid;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_KEY))
		return;

	bo_key_invalid = (tbm_key)TBM_UT_INVALID_UINT_VALUE;
	bo = tbm_bo_import(bufmgr, bo_key_invalid);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STRNE(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoImportTwice)
{
	tbm_bo bo1, bo2, bo3;
	tbm_key bo_key;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_KEY))
		return;

	bo1 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo1, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_key = tbm_bo_export(bo1);
	EXPECT_GT(bo_key, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo2 = tbm_bo_import(bufmgr, bo_key);
	EXPECT_NE(bo2, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo3 = tbm_bo_import(bufmgr, bo_key);
	EXPECT_NE(bo3, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo3);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_import_fd() */

TEST_F(TBMBo, BoImportFd)
{
	tbm_bo bo1, bo2;
	tbm_fd bo_fd;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_FD))
		return;

	bo1 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo1, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_fd = tbm_bo_export_fd(bo1);
	EXPECT_GT(bo_fd, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo2 = tbm_bo_import_fd(bufmgr, bo_fd);
	EXPECT_NE(bo2, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoImportFdWithBufmgrParamTest)
{
	tbm_bo bo;
	tbm_fd bo_fd = (tbm_fd)TBM_UT_INVALID_UINT_VALUE;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_FD))
		return;

	bo = tbm_bo_import_fd(NULL, bo_fd);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	bo = tbm_bo_import_fd(invalid_bufmgr, bo_fd);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoImportFdWithFdParamTest)
{
	tbm_bo bo;
	tbm_fd bo_fd_invalid;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_FD))
		return;

	bo_fd_invalid = (tbm_fd)TBM_UT_INVALID_UINT_VALUE;
	bo = tbm_bo_import_fd(bufmgr, bo_fd_invalid);
	EXPECT_EQ(bo, nullptr);
	EXPECT_STRNE(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoImportFdTwice)
{
	tbm_bo bo1, bo2, bo3;
	tbm_fd bo_fd;

	if (!(bufmgr_capability & TBM_BUFMGR_CAPABILITY_SHARE_FD))
		return;

	bo1 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo1, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_fd = tbm_bo_export_fd(bo1);
	EXPECT_GT(bo_fd, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo2 = tbm_bo_import_fd(bufmgr, bo_fd);
	EXPECT_NE(bo2, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo3 = tbm_bo_import_fd(bufmgr, bo_fd);
	EXPECT_NE(bo3, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo3);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_size() */

TEST_F(TBMBo, BoSize)
{
	tbm_bo bo;
	int size = 0;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	size = tbm_bo_size(bo);
	EXPECT_GT(size, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoSizeWithBoParamTest)
{
	int size = 0;

	size = tbm_bo_size(NULL);
	EXPECT_EQ(size, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	size = tbm_bo_size(invalid_bo);
	EXPECT_EQ(size, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_bo_locked() */

TEST_F(TBMBo, BoLocked)
{
	tbm_bo bo;
	tbm_bo_handle bo_handle;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* unlocked */
	ret = tbm_bo_locked(bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo_handle = tbm_bo_map(bo, TBM_DEVICE_CPU, TBM_OPTION_READ);
	EXPECT_NE(bo_handle.ptr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* locked */
	ret = tbm_bo_locked(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_unmap(bo);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* unlocked */
	ret = tbm_bo_locked(bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoLockedWithBoParamTest)
{
	int ret;

	ret = tbm_bo_locked(NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_locked(invalid_bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

#if 0
TEST_F(TBMBo, BoLockedOnce)
{
	// TODO: need to be fixed. use tbm_bufmgr_set_bo_lock_type instead of BUFMGR_LOCK_TYPE
	tbm_bo_handle bo_handle1, bo_handle2;
	tbm_bo bo;
	int ret_locked;

	setenv("BUFMGR_LOCK_TYPE", "once", 1);

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, bo);

	bo_handle1 = tbm_bo_map(bo, TBM_DEVICE_2D, TBM_OPTION_READ | TBM_OPTION_WRITE);
	bo_handle2 = tbm_bo_map(bo, TBM_DEVICE_2D, TBM_OPTION_READ | TBM_OPTION_WRITE);
	(void) bo_handle1;
	(void) bo_handle2;

	tbm_bo_unmap(bo);

	ret_locked = tbm_bo_locked(bo);

	tbm_bo_unmap(bo);

	EXPECT_EQ(ret_locked, 0);
}

TEST_F(TBMBo, BoLockedAlways)
{
	// TODO: need to be fixed. use tbm_bufmgr_set_bo_lock_type instead of BUFMGR_LOCK_TYPE
	tbm_bo_handle bo_handle1, bo_handle2;
	tbm_bo bo;
	int ret_locked;

	setenv("BUFMGR_LOCK_TYPE", "always", 1);

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, bo);

	bo_handle1 = tbm_bo_map(bo, TBM_DEVICE_2D, TBM_OPTION_READ | TBM_OPTION_WRITE);
	bo_handle2 = tbm_bo_map(bo, TBM_DEVICE_2D, TBM_OPTION_READ | TBM_OPTION_WRITE);
	(void) bo_handle1;
	(void) bo_handle2;

	tbm_bo_unmap(bo);

	ret_locked = tbm_bo_locked(bo);

	tbm_bo_unmap(bo);

	EXPECT_EQ(ret_locked, 1);
}

TEST_F(TBMBo, BoLockedNone)
{
	// TODO: need to be fixed. use tbm_bufmgr_set_bo_lock_type instead of BUFMGR_LOCK_TYPE
	tbm_bo_handle bo_handle1;
	tbm_bo bo;
	int ret_locked;

	setenv("BUFMGR_LOCK_TYPE", "none", 1);

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, bo);

	bo_handle1 = tbm_bo_map(bo, TBM_DEVICE_2D, TBM_OPTION_READ | TBM_OPTION_WRITE);
	(void) bo_handle1;

	ret_locked = tbm_bo_locked(bo);

	tbm_bo_unmap(bo);

	EXPECT_EQ(ret_locked, 0);
}
#endif

/* tbm_bo_swap() */

TEST_F(TBMBo, BoSwap)
{
	tbm_bo bo1, bo2;
	int ret;

	bo1 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo1, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo2 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo2, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_swap(bo1, bo2);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoSwapWithBo1Bo2ParamTest)
{
	tbm_bo bo;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_swap(NULL, NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	ret = tbm_bo_swap(bo, NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	ret = tbm_bo_swap(NULL, bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_swap(invalid_bo, invalid_bo2);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	ret = tbm_bo_swap(bo, invalid_bo2);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	ret = tbm_bo_swap(invalid_bo, bo);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	ret = tbm_bo_swap(invalid_bo, NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	ret = tbm_bo_swap(NULL, invalid_bo2);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoSwapWithDifferentSizedBos)
{
	tbm_bo bo1, bo2;
	int ret;

	bo1 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	bo2 = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE + 256, TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_swap(bo1, bo2);
	EXPECT_EQ(ret, 0);
	EXPECT_STRNE(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_add_user_data() */

TEST_F(TBMBo, BoAddUserData)
{
	tbm_bo bo;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_add_user_data(bo, 4887, NULL);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoAddUserDataWithBoParamTest)
{
	int ret;

	ret = tbm_bo_add_user_data(NULL, 4887, NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_add_user_data(invalid_bo, 4887, NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoAddUserDataSameKeyTwiceTest)
{
	tbm_bo bo;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_add_user_data(bo, 4887, NULL);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_add_user_data(bo, 4887, NULL);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_delete_user_data() */

TEST_F(TBMBo, BoDeleteUserData)
{
	tbm_bo bo;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);

	ret = tbm_bo_add_user_data(bo, 4887, NULL);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_delete_user_data(bo, 4887);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoDeleteUserDataWithBoParam)
{
	int ret;

	ret = tbm_bo_delete_user_data(NULL, 4887);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_delete_user_data(invalid_bo, 4887);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoDeleteUserDataWithKeyParam)
{
	tbm_bo bo;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* no use_data in bo */
	ret = tbm_bo_delete_user_data(bo, 4887);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	/* delete the data with the wrong key */
	ret = tbm_bo_add_user_data(bo, 4887, NULL);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bo_delete_user_data(bo, 4887 + 10);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_set_user_data() */

static void
_tc_tbm_bo_user_data_free(void *user_data)
{
	int *i = (int *)user_data;
	*i = 1;
}

TEST_F(TBMBo, BoSetUserData)
{
	tbm_bo bo;
	unsigned long key = 4887;
	int data1 = 0, data2 = 0;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_add_user_data(bo, key, _tc_tbm_bo_user_data_free);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_set_user_data(bo, key, &data1);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_set_user_data(bo, key, &data2);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_delete_user_data(bo, key);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	EXPECT_TRUE(data1 == 1);

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoSetUserDataWithBoParamTest)
{
	unsigned long key = 4887;
	int data1 = 0, data2 = 0;
	int ret;

	ret = tbm_bo_set_user_data(NULL, key, &data1);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_set_user_data(invalid_bo, key, &data2);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoSetUserDataWithKeyParamTest)
{
	tbm_bo bo;
	unsigned long key = 4887;
	int data = 0;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* no add_user_data */
	ret = tbm_bo_set_user_data(bo, 4887, &data);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_add_user_data(bo, key, _tc_tbm_bo_user_data_free);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* with non-existed key */
	ret = tbm_bo_set_user_data(bo, key + 10, &data);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_delete_user_data(bo, key);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_get_user_data() */

TEST_F(TBMBo, BoGetUserData)
{
	tbm_bo bo;
	unsigned long key = 4887;
	int data1 = 0;
	int *data2;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_add_user_data(bo, key, NULL);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_set_user_data(bo, key, &data1);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* success */
	ret = tbm_bo_get_user_data(bo, key, (void **)&data2);
	EXPECT_EQ(ret, 1);
	EXPECT_TRUE(data2 == &data1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	ret = tbm_bo_delete_user_data(bo, key);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoGetUserDataWithBoParamTest)
{
	unsigned long key = 4887;
	int *data1;
	int *data2;
	int ret;

	ret = tbm_bo_get_user_data(NULL, key, (void **)&data1);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_get_user_data(invalid_bo, key, (void **)&data2);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBo, BoGetUserDataWithKeyParamTest)
{
	tbm_bo bo;
	unsigned long key = 4887;
	int *data;
	int ret;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, TBM_BO_DEFAULT);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* no add_user_data/set_user_data */
	ret = tbm_bo_get_user_data(bo, 4887, (void **)&data);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_add_user_data(bo, key, _tc_tbm_bo_user_data_free);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* with non-existed key */
	ret = tbm_bo_get_user_data(bo, key + 10, (void **)&data);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	ret = tbm_bo_delete_user_data(bo, key);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	/* removed data */
	ret = tbm_bo_get_user_data(bo, key, (void **)&data);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_bo_get_flags() */

TEST_F(TBMBo, BoGetFlags)
{
	tbm_bo bo;
	int flags1 = TBM_BO_DEFAULT, flags2 = -1;

	bo = tbm_bo_alloc(bufmgr, TBM_UT_BO_SIZE, flags1);
	EXPECT_NE(bo, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	flags2 = tbm_bo_get_flags(bo);
	EXPECT_TRUE(flags1 == flags2);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bo_unref(bo);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBo, BoGetFlagsWithBoParamTest)
{
	int flags = -1;

	flags = tbm_bo_get_flags(NULL);
	EXPECT_TRUE(flags == 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	flags = tbm_bo_get_flags(invalid_bo);
	EXPECT_TRUE(flags == 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

INSTANTIATE_TEST_CASE_P(TBMBoParams,
						TBMBo,
						Combine(Bool(), Bool(), Values("none")));
