
/**************************************************************************
 *
 * Copyright 2016 Samsung Electronics co., Ltd. All Rights Reserved.
 *
 * Contact: Konstantin Drabeniuk <k.drabeniuk@samsung.com>
 * Contact: Andrii Sokolenko <a.sokolenko@samsung.com>
 * Contact: Roman Marchenko <r.marchenko@samsung.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
**************************************************************************/

#include "tc_tbm.h"
class TBMSurfaceQueue : public TBMBufmgr
{
protected:
	void SetUp();
	void TearDown();
public:
	int width;
	int height;
	int format;

	uint32_t num_formats;
	tbm_format *formats;

	tbm_surface_h alien_surface;
	tbm_surface_queue_h queue;

	tbm_surface_queue_error_e result;
	tbm_surface_h surface;
};

void TBMSurfaceQueue::SetUp()
{
	tbm_surface_error_e result = TBM_SURFACE_ERROR_NONE;

	TBMBufmgr::SetUp();

	result = (tbm_surface_error_e)tbm_surface_query_formats(&formats, &num_formats);
	EXPECT_STREQ(tbm_error_str(TBM_ERROR_NONE), tbm_error_str(result));

	for (uint32_t i = 0; i < num_formats; i++)
		if (formats[i] == TBM_FORMAT_ARGB8888)
			format = TBM_FORMAT_ARGB8888;

	/* fail if there is no TBM_FORMAT_ARGB8888 format */
	ASSERT_TRUE(format == TBM_FORMAT_ARGB8888);

	width = 720;
	height = 1024;
	alien_surface = tbm_surface_create(width, height, format);
	EXPECT_NE((tbm_surface_h)NULL, alien_surface);

	queue = tbm_surface_queue_create(TBM_UT_SURFACE_QUEUE_SIZE, width, height, format, TBM_BO_DEFAULT);
	EXPECT_NE((tbm_surface_queue_h)NULL, queue);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

void TBMSurfaceQueue::TearDown()
{
	if (alien_surface)
		tbm_surface_destroy(alien_surface);

	if (queue)
		tbm_surface_queue_destroy(queue);

	if (formats)
		free(formats);

	TBMBufmgr::TearDown();
}

static int memory_for_invalid_param;
static tbm_surface_queue_h invalid_queue = (tbm_surface_queue_h)&memory_for_invalid_param;
static tbm_surface_h invalid_surface = (tbm_surface_h)&memory_for_invalid_param;

static int cb_data;
static void tbm_surface_queue_event_handler(tbm_surface_queue_h surface_queue, void *data)
{
	if (data == &cb_data)
		*((int *)data) = 1;
}

static tbm_surface_queue_trace trace_cb_data;
static void tbm_surface_queue_trace_event_handler(tbm_surface_queue_h surface_queue,
		tbm_surface_h surface, tbm_surface_queue_trace trace, void *data)
{
	if (data == &trace_cb_data)
		trace_cb_data = trace;
}

static tbm_surface_h tbm_surface_alloc_cb_handler(tbm_surface_queue_h surface_queue, void *data)
{
	return (tbm_surface_h)data;
}

void tbm_surface_free_cb_handler(tbm_surface_queue_h surface_queue, void *data, tbm_surface_h surface)
{
	if (data == surface)
		tbm_surface_destroy(surface);
}

TEST_F(TBMSurfaceQueue, CreateDestroyQueue)
{
	tbm_surface_queue_h queue1 = NULL;

	queue1 = tbm_surface_queue_create(TBM_UT_SURFACE_QUEUE_SIZE, 720, 1024, format, TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_surface_queue_destroy(queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_surface_queue_h tbm_surface_queue_create(int TBM_UT_SURFACE_QUEUE_SIZE, int width,
		int height, int format, int flags); */
TEST_F(TBMSurfaceQueue, CreateFailInvalidInput)
{
	tbm_surface_queue_h queue1 = NULL;

	queue1 = tbm_surface_queue_create(-1, 720, 1024, TBM_FORMAT_ARGB8888, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1); // Expected Value: NULL
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	queue1 = tbm_surface_queue_create(TBM_UT_SURFACE_QUEUE_SIZE, -1, 1024, TBM_FORMAT_ARGB8888, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	queue1 = tbm_surface_queue_create(TBM_UT_SURFACE_QUEUE_SIZE, 720, -1, TBM_FORMAT_ARGB8888, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	queue1 = tbm_surface_queue_create(TBM_UT_SURFACE_QUEUE_SIZE, 720, 1024, 0, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_surface_queue_h tbm_surface_queue_sequence_create(int TBM_UT_SURFACE_QUEUE_SIZE, int width,
		int height, int format, int flags); */
TEST_F(TBMSurfaceQueue, SequenceCreateFailInvalidInput)
{
	tbm_surface_queue_h queue1 = NULL;

	queue1 = tbm_surface_queue_sequence_create(-1, 720, 1024, TBM_FORMAT_ARGB8888, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1); // Expected Value: NULL
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	queue1 = tbm_surface_queue_sequence_create(TBM_UT_SURFACE_QUEUE_SIZE, -1, 1024, TBM_FORMAT_ARGB8888, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	queue1 = tbm_surface_queue_sequence_create(TBM_UT_SURFACE_QUEUE_SIZE, 720, -1, TBM_FORMAT_ARGB8888, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	queue1 = tbm_surface_queue_sequence_create(TBM_UT_SURFACE_QUEUE_SIZE, 720, 1024, 0, TBM_BO_DEFAULT);
	EXPECT_EQ(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, CreateDestroySequenceQueue)
{
	tbm_surface_queue_h queue1 = NULL;

	queue1 = tbm_surface_queue_sequence_create(TBM_UT_SURFACE_QUEUE_SIZE, 720, 1024, formats[0], TBM_BO_DEFAULT);
	EXPECT_NE(nullptr, queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_surface_queue_destroy(queue1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}


/* void tbm_surface_queue_destroy(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, DestroyNULL)
{
	tbm_surface_queue_destroy(NULL);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, DestroyTwice)
{
	tbm_surface_queue_destroy(queue);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_surface_queue_destroy(queue);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	/* to prevent destroy in TearDown */
	queue = NULL;
}

/* tbm_surface_queue_error_e tbm_surface_queue_dequeue(
		tbm_surface_queue_h surface_queue, tbm_surface_h *surface); */
TEST_F(TBMSurfaceQueue, DequeueFailNull)
{
	ASSERT_EXIT({
		result = tbm_surface_queue_dequeue(NULL, &surface);
		EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

		result = tbm_surface_queue_dequeue(queue, NULL);
		EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

		exit(0);
	}, ::testing::ExitedWithCode(0), "");
}

TEST_F(TBMSurfaceQueue, DequeueFailInvaildInput)
{
	result = tbm_surface_queue_dequeue(invalid_queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, DequeueSuccess)
{
	/* do dequeue three times */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		EXPECT_NE((tbm_surface_h)NULL, surface);
	}

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(nullptr, surface);
}

/* tbm_surface_queue_error_e tbm_surface_queue_get_surfaces( tbm_surface_queue_h surface_queue,
	tbm_surface_h *surfaces, int *num); */
TEST_F(TBMSurfaceQueue, GetSurfacesFailNull)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE];
	int num;

	result = tbm_surface_queue_get_surfaces(NULL, surfaces, &num);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_get_surfaces(queue, surfaces, NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, GetSurfacesFailInvaildInput)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE];
	int num;

	result = tbm_surface_queue_get_surfaces(invalid_queue, surfaces, &num);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, GetSurfacesSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE];
	int num = 0;

	/* test: empty queue*/
	result = tbm_surface_queue_get_surfaces(queue, surfaces, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(0, num);

	/* do dequeue three times */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		EXPECT_NE(nullptr, surface);
	}

	/* test */
	result = tbm_surface_queue_get_surfaces(queue, surfaces, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(TBM_UT_SURFACE_QUEUE_SIZE, num);
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++)
		EXPECT_NE(nullptr, surfaces[i]);

	/* test: get only number of surfaces */
	result = tbm_surface_queue_get_surfaces(queue, NULL, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(TBM_UT_SURFACE_QUEUE_SIZE, num);
}

/* tbm_surface_queue_error_e tbm_surface_queue_cancel_dequeue(
		tbm_surface_queue_h surface_queue, tbm_surface_h surface); */
TEST_F(TBMSurfaceQueue, CancelDequeueFailNull)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_dequeue(NULL, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_dequeue(queue, NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, CancelDequeueFailInvaildInput)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));


	result = tbm_surface_queue_cancel_dequeue(invalid_queue, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_dequeue(queue, invalid_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_dequeue(queue, alien_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, CancelDequeueSuccess)
{
	tbm_surface_h last_surface = NULL;

	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		EXPECT_NE(nullptr, surface);
	}

	last_surface = surface;
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(nullptr, surface);

	/* test */
	result = tbm_surface_queue_cancel_dequeue(queue, last_surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(last_surface, surface);
}

/* tbm_surface_queue_error_e tbm_surface_queue_enqueue(
		tbm_surface_queue_h surface_queue, tbm_surface_h surface); */
TEST_F(TBMSurfaceQueue, EnqueueFailNull)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_enqueue(NULL, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_enqueue(this->queue, NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, EnqueueFailInvaildInput)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_enqueue(invalid_queue, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_enqueue(queue, invalid_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_enqueue(queue, alien_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, EnqueueSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE] = { NULL };


	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_enqueue(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}
}

TEST_F(TBMSurfaceQueue, EnqueueFailTwice)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

/* tbm_surface_queue_error_e tbm_surface_queue_acquire(
		tbm_surface_queue_h surface_queue, tbm_surface_h *surface); */
TEST_F(TBMSurfaceQueue, AcquireFailNull)
{
	/* test */
	result = tbm_surface_queue_acquire(NULL, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AcquireFailInvaildInput)
{
	result = tbm_surface_queue_acquire(invalid_queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AcquireFailAcquireEmptyQueue)
{
	result = tbm_surface_queue_acquire(invalid_queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AcquireSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE] = { NULL };
	tbm_surface_h acquired_surface = NULL;


	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_enqueue(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_acquire(queue, &acquired_surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		EXPECT_EQ(surfaces[i], acquired_surface);
	}

	/* test: acquire empty queue*/
	result = tbm_surface_queue_acquire(queue, &acquired_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(nullptr, acquired_surface);
}

/* tbm_surface_queue_error_e tbm_surface_queue_cancel_acquire(
		tbm_surface_queue_h surface_queue, tbm_surface_h surface); */
TEST_F(TBMSurfaceQueue, CancelAcquireFailNull)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);

	/* test */
	result = tbm_surface_queue_cancel_acquire(NULL, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_acquire(queue, NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, CancelAcquireFailInvaildInput)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_cancel_acquire(invalid_queue, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_acquire(queue, invalid_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_cancel_acquire(queue, alien_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, CancelAcquireSuccess)
{
	tbm_surface_h last_surface = NULL;

	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_enqueue(queue, surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_acquire(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	last_surface = surface;
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(nullptr, surface);

	/* test */
	result = tbm_surface_queue_cancel_acquire(queue, last_surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(last_surface, surface);
}

/* tbm_surface_queue_error_e tbm_surface_queue_release(
		tbm_surface_queue_h surface_queue, tbm_surface_h surface); */
TEST_F(TBMSurfaceQueue, ReleaseFailNull)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_release(NULL, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_release(this->queue, NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, ReleaseFailInvaildInput)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_release(invalid_queue, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_release(queue, invalid_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_release(queue, alien_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, ReleaseSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE] = { NULL };

	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_enqueue(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_acquire(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_release(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}
}

TEST_F(TBMSurfaceQueue, ReleaseSequenceSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE] = { NULL };

	/* create a sequence queue instead of a default one*/
	tbm_surface_queue_destroy(queue);
	queue = tbm_surface_queue_sequence_create(TBM_UT_SURFACE_QUEUE_SIZE, width, height, format, TBM_BO_DEFAULT);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(tbm_get_last_error()));

	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_enqueue(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_acquire(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_release(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}
}

TEST_F(TBMSurfaceQueue, ReleaseFailTwice)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_release(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_release(queue, surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

/* int tbm_surface_queue_get_width(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, GetWidthFailNull)
{
	int width = tbm_surface_queue_get_width(NULL);
	EXPECT_EQ(0, width);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetWidthFailInvalidInput)
{
	int width = tbm_surface_queue_get_width(invalid_queue);
	EXPECT_EQ(0, width);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetWidthSuccess)
{
	int width = tbm_surface_queue_get_width(queue);
	EXPECT_EQ(this->width, width);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* int tbm_surface_queue_get_height(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, GetHeightFailNull)
{
	int height = tbm_surface_queue_get_height(NULL);
	EXPECT_EQ(0, height);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetHeightFailInvalidInput)
{
	int height = tbm_surface_queue_get_height(invalid_queue);
	EXPECT_EQ(0, height);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetHeightSuccess)
{
	int height = tbm_surface_queue_get_height(queue);
	EXPECT_EQ(this->height, height);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* int tbm_surface_queue_get_format(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, GetFormatFailNull)
{
	int format = tbm_surface_queue_get_format(NULL);
	EXPECT_EQ(0, format);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetFormatFailInvalidInput)
{
	int format = tbm_surface_queue_get_format(invalid_queue);
	EXPECT_EQ(0, format);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetFormatSuccess)
{
	int format = tbm_surface_queue_get_format(queue);
	EXPECT_EQ((int)this->format, format);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* int tbm_surface_queue_get_size(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, GetSizeFailNull)
{
	int q_size = tbm_surface_queue_get_size(NULL);
	EXPECT_EQ(0, q_size);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetSizeFailInvalidInput)
{
	int q_size = tbm_surface_queue_get_size(invalid_queue);
	EXPECT_EQ(0, q_size);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMSurfaceQueue, GetSizeSuccess)
{
	int q_size = tbm_surface_queue_get_size(queue);
	EXPECT_EQ(TBM_UT_SURFACE_QUEUE_SIZE,  q_size);
	EXPECT_EQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

/* tbm_surface_queue_error_e tbm_surface_queue_reset(
		tbm_surface_queue_h surface_queue, int width, int height, int format); */
TEST_F(TBMSurfaceQueue, ResetFailNull)
{
	result = tbm_surface_queue_reset(NULL, width, height, format);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, ResetFailInvalidInput)
{
	result = tbm_surface_queue_reset(invalid_queue, width, height, format);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_reset(invalid_queue, -1, height, format);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_reset(invalid_queue, width, -1, format);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_reset(invalid_queue, width, height, 0);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, ResetSuccessEmptyQueue)
{
	result = tbm_surface_queue_reset(queue, width-10, height-10, format);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, ResetSuccessSameParams)
{
	tbm_surface_h surface = NULL;


	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_reset(queue, width, height, format);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	ASSERT_TRUE(tbm_surface_internal_is_valid(surface)) << "old surface mast be valid";
}

TEST_F(TBMSurfaceQueue, ResetSuccess)
{
	tbm_surface_h surface = NULL;


	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_reset(queue, width-10, height-10, formats[num_formats-1]);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	ASSERT_FALSE(tbm_surface_internal_is_valid(surface)) << "old surface has to be deleted";
}

TEST_F(TBMSurfaceQueue, ResetSequenceSuccess)
{
	tbm_surface_h surface = NULL;

	/* create a sequence queue instead of a default one*/
	tbm_surface_queue_destroy(queue);
	queue = tbm_surface_queue_sequence_create(TBM_UT_SURFACE_QUEUE_SIZE, width, height, format, TBM_BO_DEFAULT);
	ASSERT_TRUE(queue != NULL);

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_reset(queue, width-10, height-10, formats[num_formats-1]);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	ASSERT_FALSE(tbm_surface_internal_is_valid(surface)) << "old surface has to be deleted";
}

/* tbm_surface_queue_error_e tbm_surface_queue_set_size(
		tbm_surface_queue_h surface_queue, int TBM_UT_SURFACE_QUEUE_SIZE, int flush); */
TEST_F(TBMSurfaceQueue, SetSizeFailNull)
{
	result = tbm_surface_queue_set_size(NULL, TBM_UT_SURFACE_QUEUE_SIZE, 0);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetSizeFailInvalidInput)
{
	result = tbm_surface_queue_set_size(invalid_queue, TBM_UT_SURFACE_QUEUE_SIZE, 0);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_set_size(NULL, 0, 0);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetSizeSuccessTheSameSize)
{
	tbm_surface_h surface = NULL;

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/*test: */
	result = tbm_surface_queue_set_size(queue, TBM_UT_SURFACE_QUEUE_SIZE, 0);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(tbm_surface_internal_is_valid(surface)) << "old surface mast be valid";

	/*test: with flush*/
	result = tbm_surface_queue_set_size(queue, TBM_UT_SURFACE_QUEUE_SIZE, 1);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_FALSE(tbm_surface_internal_is_valid(surface)) << "old surface has to be deleted";
}

TEST_F(TBMSurfaceQueue, SetSizeSuccessIncrease)
{
	tbm_surface_h surface = NULL;

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/*test: */
	result = tbm_surface_queue_set_size(queue, TBM_UT_SURFACE_QUEUE_SIZE+1, 0);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(tbm_surface_internal_is_valid(surface)) << "old surface mast be valid";

	/*test: with flush*/
	result = tbm_surface_queue_set_size(queue, TBM_UT_SURFACE_QUEUE_SIZE+2, 1);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_FALSE(tbm_surface_internal_is_valid(surface)) << "old surface has to be deleted";
}

TEST_F(TBMSurfaceQueue, SetSizeSuccessReduce)
{
	tbm_surface_h surface = NULL;

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/*test: */
	result = tbm_surface_queue_set_size(queue, TBM_UT_SURFACE_QUEUE_SIZE-1, 0);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(tbm_surface_internal_is_valid(surface)) << "old surface mast be valid";

	/*test: with flush*/
	result = tbm_surface_queue_set_size(queue, TBM_UT_SURFACE_QUEUE_SIZE-2, 1);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_FALSE(tbm_surface_internal_is_valid(surface)) << "old surface has to be deleted";
}

/* tbm_surface_queue_error_e tbm_surface_queue_flush(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, FlushFailNull)
{
	result = tbm_surface_queue_flush(NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, FlushFailInvalidInput)
{
	result = tbm_surface_queue_flush(invalid_queue);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, FlushSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE];

	/*test: flushing not initialized queue*/
	result = tbm_surface_queue_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));


	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_enqueue(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	result = tbm_surface_queue_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++)
		ASSERT_FALSE(tbm_surface_internal_is_valid(surfaces[i])) << "old surface has to be deleted";
}

/* tbm_surface_queue_error_e tbm_surface_queue_free_flush(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, FreeFlushFailNull)
{
	result = tbm_surface_queue_free_flush(NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, FreeFlushFailInvalidInput)
{
	result = tbm_surface_queue_free_flush(invalid_queue);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, FreeFlushSuccess)
{
	tbm_surface_h surfaces[TBM_UT_SURFACE_QUEUE_SIZE];

	/*test: flushing not initialized queue*/
	result = tbm_surface_queue_free_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));


	/* init queue */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++) {
		result = tbm_surface_queue_dequeue(queue, &surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_cancel_dequeue(queue, surfaces[i]);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	result = tbm_surface_queue_free_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* extra check */
	for (int i = 0; i < TBM_UT_SURFACE_QUEUE_SIZE; i++)
		ASSERT_FALSE(tbm_surface_internal_is_valid(surfaces[i])) << "old surface has to be deleted";
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_reset_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb reset_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddResetCbFailNull)
{
	result = tbm_surface_queue_add_reset_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_reset_cb(queue, NULL, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddResetCbFailInvalidInput)
{
	result = tbm_surface_queue_add_reset_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddResetCbSuccess)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_add_reset_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/*extra check: reset cb has to called after tbm_surface_queue_flush()*/
	cb_data = 0;
	result = tbm_surface_queue_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}

/* tbm_surface_queue_error_e tbm_surface_queue_remove_reset_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb reset_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveResetCbFailNull)
{
	result = tbm_surface_queue_remove_reset_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveResetCbSuccess)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	result = tbm_surface_queue_enqueue(queue, surface);
	result = tbm_surface_queue_add_reset_cb(queue, tbm_surface_queue_event_handler, &cb_data);

	/* test */
	result = tbm_surface_queue_remove_reset_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/*extra check: reset cb musn't called after tbm_surface_queue_flush()*/
	cb_data = 0;
	result = tbm_surface_queue_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 0);
}

TEST_F(TBMSurfaceQueue, RemoveResetCbFailInvalidInput)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	result = tbm_surface_queue_enqueue(queue, surface);
	result = tbm_surface_queue_add_reset_cb(queue, tbm_surface_queue_event_handler, &cb_data);

	/* test: invalid queue*/
	result = tbm_surface_queue_remove_reset_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test: invalid data */
	result = tbm_surface_queue_remove_reset_cb(queue, tbm_surface_queue_event_handler, NULL);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test: invalid cb function */
	result = tbm_surface_queue_remove_reset_cb(queue, (tbm_surface_queue_notify_cb)1, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/*extra check: reset cb still has to be called after tbm_surface_queue_flush()*/
	cb_data = 0;
	result = tbm_surface_queue_flush(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}

/* tbm_surface_queue_error_e tbm_surface_queue_notify_reset(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, NotifyResetFailNull)
{
	result = tbm_surface_queue_notify_reset(NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, NotifyResetFailInvalidInput)
{
	result = tbm_surface_queue_notify_reset(invalid_queue);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, NotifyResetCbSuccess)
{
	result = tbm_surface_queue_add_reset_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	cb_data = 0;
	result = tbm_surface_queue_notify_reset(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}

/* tbm_surface_queue_error_e tbm_surface_queue_notify_dequeuable(tbm_surface_queue_h surface_queue); */
TEST_F(TBMSurfaceQueue, NotifyDequeuableFailNull)
{
	result = tbm_surface_queue_notify_dequeuable(NULL);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, NotifyDequeuableFailInvalidInput)
{
	result = tbm_surface_queue_notify_dequeuable(invalid_queue);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, NotifyDequeuableCbSuccess)
{
	/* test */
	result = tbm_surface_queue_add_dequeuable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_notify_dequeuable(queue);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}

/* int tbm_surface_queue_can_dequeue(tbm_surface_queue_h surface_queue, int wait); */
TEST_F(TBMSurfaceQueue, CanDequeueFailNull)
{
	ASSERT_FALSE(tbm_surface_queue_can_dequeue(NULL, 0));
}

TEST_F(TBMSurfaceQueue, CanDequeueFailInvalidInput)
{
	ASSERT_FALSE(tbm_surface_queue_can_dequeue(invalid_queue, 0));
}

TEST_F(TBMSurfaceQueue, CanDequeueSuccess)
{
	int dq_count = 0;

	/* test: dequeue all*/
	while (tbm_surface_queue_can_dequeue(queue, 0)) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		dq_count++;
	}
	EXPECT_EQ(TBM_UT_SURFACE_QUEUE_SIZE, dq_count);
}

TEST_F(TBMSurfaceQueue, CanDequeueFailEmptyQueue)
{
	/*Dequeue all*/
	while (tbm_surface_queue_can_dequeue(queue, 0)) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	}

	/* test */
	ASSERT_FALSE(tbm_surface_queue_can_dequeue(queue, 0));
}


/* int tbm_surface_queue_can_acquire(tbm_surface_queue_h surface_queue, int wait); */
TEST_F(TBMSurfaceQueue, CanAcquireFailNull)
{
	ASSERT_FALSE(tbm_surface_queue_can_acquire(NULL, 0));
}

TEST_F(TBMSurfaceQueue, CanAcquireFailInvalidInput)
{
	ASSERT_FALSE(tbm_surface_queue_can_acquire(invalid_queue, 0));
}

TEST_F(TBMSurfaceQueue, CanAcquireFailEmptyQueue)
{
	ASSERT_FALSE(tbm_surface_queue_can_acquire(queue, 0));
}

TEST_F(TBMSurfaceQueue, CanAcquireSuccess)
{
	int acq_count = 0;

	while (tbm_surface_queue_can_dequeue(queue, 0)) {
		result = tbm_surface_queue_dequeue(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		result = tbm_surface_queue_enqueue(queue, surface);
	}

	/*Dequeue all*/
	while (tbm_surface_queue_can_acquire(queue, 0)) {
		result = tbm_surface_queue_acquire(queue, &surface);
		EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
		acq_count++;
	}

	EXPECT_EQ(TBM_UT_SURFACE_QUEUE_SIZE, acq_count);
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_destroy_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb destroy_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddDestroyCbFailNull)
{
	result = tbm_surface_queue_add_destroy_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_destroy_cb(queue, NULL, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddDestroyCbFailInvalidInput)
{
	result = tbm_surface_queue_add_destroy_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddDestroyCbSuccess)
{
	/* test */
	result = tbm_surface_queue_add_destroy_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	tbm_surface_queue_destroy(queue);
	ASSERT_TRUE(cb_data == 1);

	/* to avoid destroying queue in TearDown*/
	queue = NULL;
}

/* tbm_surface_queue_error_e tbm_surface_queue_remove_destroy_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb destroy_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveDestroyCbFailNull)
{
	result = tbm_surface_queue_remove_destroy_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveDestroyCbFailInvalidInput)
{
	result = tbm_surface_queue_remove_destroy_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveDestroyCbSuccess)
{
	result = tbm_surface_queue_add_destroy_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_remove_destroy_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	tbm_surface_queue_destroy(queue);
	ASSERT_TRUE(cb_data == 0);

	/* to avoid destroying queue in TearDown*/
	queue = NULL;
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_dequeuable_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb dequeuable_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddDequeuableCbFailNull)
{
	result = tbm_surface_queue_add_dequeuable_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_dequeuable_cb(queue, NULL, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddDequeuableCbFailInvalidInput)
{
	result = tbm_surface_queue_add_dequeuable_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddDequeuableCbSuccess)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_add_dequeuable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_release(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}


/* tbm_surface_queue_error_e tbm_surface_queue_remove_dequeuable_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb dequeuable_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveDequeuableCbFailNull)
{
	result = tbm_surface_queue_remove_dequeuable_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveDequeuableCbFailInvalidInput)
{
	/* test: invalid queue */
	result = tbm_surface_queue_remove_dequeuable_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveDequeuableCbSuccess)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_add_dequeuable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_remove_dequeuable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_release(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 0);
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_dequeue_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb dequeue_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddDequeueCbFailNull)
{
	result = tbm_surface_queue_add_dequeue_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_dequeue_cb(queue, NULL, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddDequeueCbFailInvalidInput)
{
	result = tbm_surface_queue_add_dequeue_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddDequeueCbSuccess)
{
	/* test */
	result = tbm_surface_queue_add_dequeue_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}

/* tbm_surface_queue_error_e tbm_surface_queue_remove_dequeue_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb dequeue_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveDequeueCbFailNull)
{
	result = tbm_surface_queue_remove_dequeue_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveDequeueCbFailInvalidInput)
{
	/* test: invalid queue */
	result = tbm_surface_queue_remove_dequeue_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveDequeueCbSuccess)
{
	/* test */
	result = tbm_surface_queue_add_dequeue_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);

	result = tbm_surface_queue_remove_dequeue_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 0);
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_can_dequeue_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb can_dequeue_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddCanDequeueCbFailNull)
{
	result = tbm_surface_queue_add_can_dequeue_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_can_dequeue_cb(queue, NULL, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddCanDequeueCbFailInvalidInput)
{
	result = tbm_surface_queue_add_can_dequeue_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddCanDequeueCbSuccess)
{
	/* test */
	result = tbm_surface_queue_add_can_dequeue_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	ASSERT_TRUE(tbm_surface_queue_can_dequeue(queue, 0));
	ASSERT_TRUE(cb_data == 1);
}

/* tbm_surface_queue_error_e tbm_surface_queue_remove_can_dequeue_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb can_dequeue_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveCanDequeueCbFailNull)
{
	result = tbm_surface_queue_remove_can_dequeue_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveCanDequeueCbFailInvalidInput)
{
	/* test: invalid queue */
	result = tbm_surface_queue_remove_can_dequeue_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveCanDequeueCbSuccess)
{
	result = tbm_surface_queue_add_can_dequeue_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_remove_can_dequeue_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	ASSERT_TRUE(tbm_surface_queue_can_dequeue(queue, 0));
	ASSERT_TRUE(cb_data == 0);
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_acquirable_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb acquirable_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddAcquirableCbFailNull)
{
	result = tbm_surface_queue_add_acquirable_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_acquirable_cb(queue, NULL, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddAcquirableCbFailInvalidInput)
{
	result = tbm_surface_queue_add_acquirable_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddAcquirableCbSuccess)
{
	tbm_surface_queue_dequeue(queue, &surface);

	/* test */
	result = tbm_surface_queue_add_acquirable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 1);
}

/* tbm_surface_queue_error_e tbm_surface_queue_remove_acquirable_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_notify_cb acquirable_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveAcquirableCbFailNull)
{
	result = tbm_surface_queue_remove_acquirable_cb(NULL, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveAcquirableCbFailInvalidInput)
{
	/* test: invalid queue */
	result = tbm_surface_queue_remove_acquirable_cb(invalid_queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveAcquirableCbSuccess)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	result = tbm_surface_queue_add_acquirable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_remove_acquirable_cb(queue, tbm_surface_queue_event_handler, &cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	cb_data = 0;
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(cb_data == 0);
}

/* tbm_surface_queue_error_e tbm_surface_queue_add_trace_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_trace_cb trace_cb, void *data); */
TEST_F(TBMSurfaceQueue, AddTraceCbFailNull)
{
	result = tbm_surface_queue_add_trace_cb(NULL, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_add_trace_cb(queue, NULL, &trace_cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddTraceCbFailInvalidInput)
{
	result = tbm_surface_queue_add_trace_cb(invalid_queue, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, AddTraceCbSuccess)
{
	trace_cb_data = TBM_SURFACE_QUEUE_TRACE_NONE;

	/* test */
	result = tbm_surface_queue_add_trace_cb(queue, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_DEQUEUE);

	result = tbm_surface_queue_cancel_dequeue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_CANCEL_DEQUEUE);

	result = tbm_surface_queue_dequeue(queue, &surface);
	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_ENQUEUE);

	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_ACQUIRE);

	result = tbm_surface_queue_cancel_acquire(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_CANCEL_ACQUIRE);

	result = tbm_surface_queue_acquire(queue, &surface);
	result = tbm_surface_queue_release(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_RELEASE);
}

/* tbm_surface_queue_error_e tbm_surface_queue_remove_trace_cb(
	tbm_surface_queue_h surface_queue, tbm_surface_queue_trace_cb trace_cb, void *data); */
TEST_F(TBMSurfaceQueue, RemoveTraceCbFailNull)
{
	result = tbm_surface_queue_remove_trace_cb(NULL, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveTraceCbFailInvalidInput)
{
	/* test: invalid queue */
	result = tbm_surface_queue_remove_trace_cb(invalid_queue, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, RemoveTraceCbSuccess)
{
	trace_cb_data = TBM_SURFACE_QUEUE_TRACE_NONE;

	result = tbm_surface_queue_add_trace_cb(queue, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test */
	result = tbm_surface_queue_remove_trace_cb(queue, tbm_surface_queue_trace_event_handler, &trace_cb_data);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	ASSERT_TRUE(trace_cb_data == TBM_SURFACE_QUEUE_TRACE_NONE);
}

/* tbm_surface_queue_error_e tbm_surface_queue_set_alloc_cb( tbm_surface_queue_h surface_queue,
	tbm_surface_alloc_cb alloc_cb,	tbm_surface_free_cb free_cb, void *data); */
TEST_F(TBMSurfaceQueue, SetAllocCbFailNull)
{
	result = tbm_surface_queue_set_alloc_cb(NULL, tbm_surface_alloc_cb_handler,
			tbm_surface_free_cb_handler, alien_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetAllocCbFailInvalidInput)
{
	result = tbm_surface_queue_set_alloc_cb(invalid_queue, tbm_surface_alloc_cb_handler,
			tbm_surface_free_cb_handler, alien_surface);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetAllocCbSuccess)
{
	result = tbm_surface_queue_set_alloc_cb(queue, tbm_surface_alloc_cb_handler,
			tbm_surface_free_cb_handler, alien_surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test: allocate */
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(alien_surface, surface);

	/* test:  alien_surface has to be deleted*/
	tbm_surface_queue_destroy(queue);
	ASSERT_FALSE(tbm_surface_internal_is_valid(alien_surface));

	alien_surface = NULL;
}

/* tbm_surface_queue_error_e tbm_surface_queue_get_trace_surface_num(
			tbm_surface_queue_h surface_queue, tbm_surface_queue_trace trace, int *num); */
TEST_F(TBMSurfaceQueue, GetTraceSurfaceNumFailNull)
{
	int num = 0;

	ASSERT_EXIT({

		result = tbm_surface_queue_get_trace_surface_num(NULL, TBM_SURFACE_QUEUE_TRACE_NONE, &num);
		EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

		result = tbm_surface_queue_get_trace_surface_num(queue, TBM_SURFACE_QUEUE_TRACE_NONE, NULL);
		EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

		exit(0);
	}, ::testing::ExitedWithCode(0), "");

}

TEST_F(TBMSurfaceQueue, GetTraceSurfaceNumFailInvalidInput)
{
	int num = 0;

	result = tbm_surface_queue_get_trace_surface_num(invalid_queue, TBM_SURFACE_QUEUE_TRACE_NONE, &num);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, GetTraceSurfaceNumSuccess)
{
	int num = 0;

	result = tbm_surface_queue_set_size(queue, 4, 0);

	/* one acquired */
	result = tbm_surface_queue_dequeue(queue, &surface);
	result = tbm_surface_queue_enqueue(queue, surface);
	result = tbm_surface_queue_acquire(queue, &surface);

	/* one enqueued */
	result = tbm_surface_queue_dequeue(queue, &surface);
	result = tbm_surface_queue_enqueue(queue, surface);

	/* one dequeued */
	result = tbm_surface_queue_dequeue(queue, &surface);

	/* one released */
	result = tbm_surface_queue_dequeue(queue, &surface);
	result = tbm_surface_queue_cancel_dequeue(queue, surface);


	/* test */
	result = tbm_surface_queue_get_trace_surface_num(queue, TBM_SURFACE_QUEUE_TRACE_NONE, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(0, num);

	result = tbm_surface_queue_get_trace_surface_num(queue, TBM_SURFACE_QUEUE_TRACE_DEQUEUE, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(1, num);

	result = tbm_surface_queue_get_trace_surface_num(queue, TBM_SURFACE_QUEUE_TRACE_ENQUEUE, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(1, num);

	result = tbm_surface_queue_get_trace_surface_num(queue, TBM_SURFACE_QUEUE_TRACE_ACQUIRE, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(1, num);

	result = tbm_surface_queue_get_trace_surface_num(queue, TBM_SURFACE_QUEUE_TRACE_RELEASE, &num);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
	EXPECT_EQ(1, num);
}
/* tbm_surface_queue_error_e tbm_surface_queue_set_modes(
			tbm_surface_queue_h surface_queue, int modes); */
TEST_F(TBMSurfaceQueue, SetModesFailNull)
{
	result = tbm_surface_queue_set_modes(NULL, TBM_SURFACE_QUEUE_MODE_GUARANTEE_CYCLE);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetModesFailInvalidInput)
{
	result = tbm_surface_queue_set_modes(invalid_queue, TBM_SURFACE_QUEUE_MODE_GUARANTEE_CYCLE);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetModesSuccess)
{
	result = tbm_surface_queue_set_modes(queue, TBM_SURFACE_QUEUE_MODE_GUARANTEE_CYCLE);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

/* tbm_surface_queue_set_sync_count() */
TEST_F(TBMSurfaceQueue, SetSyncCountFailNull)
{
	result = tbm_surface_queue_set_sync_count(NULL, 1);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetSyncCountFailInvalidInput)
{
	result = tbm_surface_queue_set_sync_count(invalid_queue, 1);
	EXPECT_NE(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}

TEST_F(TBMSurfaceQueue, SetSyncCountSuccess)
{
	result = tbm_surface_queue_dequeue(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test: map after enqueue */
	result = tbm_surface_queue_set_sync_count(queue, 0);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_enqueue(queue, surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	/* test: map after acquire */
	result = tbm_surface_queue_set_sync_count(queue, 0);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));

	result = tbm_surface_queue_acquire(queue, &surface);
	EXPECT_EQ(tbm_error_str(TBM_SURFACE_QUEUE_ERROR_NONE), tbm_error_str(result));
}