/**************************************************************************

libtbm

Copyright 2012 Samsung Electronics co., Ltd. All Rights Reserved.

Contact: SooChan Lim <sc1.lim@samsung.com>, Sangjin Lee <lsj119@samsung.com>
Boram Park <boram1288.park@samsung.com>, Changyeon Lee <cyeon.lee@samsung.com>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sub license, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice (including the
next paragraph) shall be included in all copies or substantial portions
of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

**************************************************************************/

#include "config.h"
#include "tbm_bufmgr.h"
#include "tbm_bufmgr_int.h"
#include "list.h"

/* check condition */
#define TBM_BO_RETURN_IF_FAIL(cond) {\
	if (!(cond)) {\
		TBM_ERR("'%s' failed.\n", #cond);\
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);\
		_tbm_bufmgr_mutex_unlock();\
		return;\
	} \
}

#define TBM_BO_RETURN_VAL_IF_FAIL(cond, val) {\
	if (!(cond)) {\
		TBM_ERR("'%s' failed.\n", #cond);\
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);\
		_tbm_bufmgr_mutex_unlock();\
		return val;\
	} \
}

char *
_tbm_flag_to_str(int f)
{
	static char str[255];

	if (f == TBM_BO_DEFAULT)
		 snprintf(str, 255, "DEFAULT");
	else {
		int c = 0;

		if (f & TBM_BO_SCANOUT)
			c += snprintf(&str[c], 255-c, "SCANOUT");

		if (f & TBM_BO_NONCACHABLE) {
			if (c >= 0 && c < 255)
				c += snprintf(&str[c], 255-c, ", ");

			if (c >= 0 && c < 255)
				c += snprintf(&str[c], 255-c, "NONCACHABLE,");
		}

		if (f & TBM_BO_WC) {
			if (c >= 0 && c < 255)
				c += snprintf(&str[c], 255-c, ", ");

			if (c >= 0 && c < 255)
				c += snprintf(&str[c], 255-c, "WC");
		}
	}

	return str;
}

/* LCOV_EXCL_STOP */

tbm_user_data *
user_data_lookup(struct list_head *user_data_list, unsigned long key)
{
	tbm_user_data *old_data = NULL;

	if (LIST_IS_EMPTY(user_data_list))
		return NULL;

	LIST_FOR_EACH_ENTRY(old_data, user_data_list, item_link) {
		if (old_data->key == key)
			return old_data;
	}

	return NULL;
}

tbm_user_data *
user_data_create(unsigned long key, tbm_data_free data_free_func)
{
	tbm_user_data *user_data;

	user_data = calloc(1, sizeof(tbm_user_data));
	if (!user_data) {
		/* LCOV_EXCL_START */
		TBM_ERR("fail to allocate an user_date\n");
		_tbm_set_last_result(TBM_ERROR_OUT_OF_MEMORY);
		return NULL;
		/* LCOV_EXCL_STOP */
	}

	user_data->key = key;
	user_data->free_func = data_free_func;

	return user_data;
}

void
user_data_delete(tbm_user_data *user_data)
{
	if (user_data->data && user_data->free_func)
		user_data->free_func(user_data->data);

	LIST_DEL(&user_data->item_link);

	free(user_data);
}

static int
_bo_lock(tbm_bo bo, int device, int opt)
{
	tbm_error_e error;
	int ret;

	error = tbm_module_bo_lock(bo->bufmgr->module, bo, bo->bo_data, device, opt);
	_tbm_set_last_result(error);
	switch(error) {
		case TBM_ERROR_NONE:
			ret = 1;
			break;
		case TBM_ERROR_NOT_SUPPORTED:
			ret = 1;
			_tbm_set_last_result(TBM_ERROR_NONE);
			break;
		default:
			ret = 0;
			TBM_WRN("fail to lock. error:%d", error);
			break;
	}

	return ret;
}

static void
_bo_unlock(tbm_bo bo)
{
	tbm_error_e error;

	error = tbm_module_bo_unlock(bo->bufmgr->module, bo, bo->bo_data);
	_tbm_set_last_result(error);
	switch(error) {
		case TBM_ERROR_NONE:
			break;
		case TBM_ERROR_NOT_SUPPORTED:
			_tbm_set_last_result(TBM_ERROR_NONE);
			break;
		default:
			TBM_WRN("fail to lock. error:%d", error);
			break;
	}
}

static int
_tbm_bo_lock(tbm_bo bo, int device, int opt)
{
	int old, ret;

	if (!bo)
		return 0;

	/* do not try to lock the bo */
	if (bo->bufmgr->bo_lock_type == TBM_BUFMGR_BO_LOCK_TYPE_NEVER)
		return 1;

	if (bo->lock_cnt < 0) {
		TBM_ERR("error bo:%p LOCK_CNT=%d\n",
			bo, bo->lock_cnt);
		return 0;
	}

	old = bo->lock_cnt;

	switch (bo->bufmgr->bo_lock_type) {
	case TBM_BUFMGR_BO_LOCK_TYPE_ONCE:
		if (bo->lock_cnt == 0) {
			_tbm_bufmgr_mutex_unlock();
			ret = _bo_lock(bo, device, opt);
			_tbm_bufmgr_mutex_lock();
			if (ret)
				bo->lock_cnt++;
		} else
			ret = 1;
		break;
	case TBM_BUFMGR_BO_LOCK_TYPE_ALWAYS:
		_tbm_bufmgr_mutex_unlock();
		ret = _bo_lock(bo, device, opt);
		_tbm_bufmgr_mutex_lock();
		if (ret)
			bo->lock_cnt++;
		break;
	default:
		TBM_ERR("error bo:%p bo_lock_type[%d] is wrong.\n",
				bo, bo->bufmgr->bo_lock_type);
		ret = 0;
		break;
	}

	TBM_DBG(">> LOCK bo:%p(%d->%d)\n", bo, old, bo->lock_cnt);

	return ret;
}

static void
_tbm_bo_unlock(tbm_bo bo)
{
	int old;

	/* do not try to unlock the bo */
	if (bo->bufmgr->bo_lock_type == TBM_BUFMGR_BO_LOCK_TYPE_NEVER)
		return;

	old = bo->lock_cnt;

	switch (bo->bufmgr->bo_lock_type) {
	case TBM_BUFMGR_BO_LOCK_TYPE_ONCE:
		if (bo->lock_cnt > 0) {
			bo->lock_cnt--;
			if (bo->lock_cnt == 0)
				_bo_unlock(bo);
		}
		break;
	case TBM_BUFMGR_BO_LOCK_TYPE_ALWAYS:
		if (bo->lock_cnt > 0) {
			bo->lock_cnt--;
			_bo_unlock(bo);
		}
		break;
	default:
		TBM_ERR("error bo:%p bo_lock_type[%d] is wrong.\n",
				bo, bo->bufmgr->bo_lock_type);
		break;
	}

	if (bo->lock_cnt < 0)
		bo->lock_cnt = 0;

	TBM_DBG(">> UNLOCK bo:%p(%d->%d)\n", bo, old, bo->lock_cnt);
}

static int
_tbm_bo_magic_check(tbm_bo bo)
{
	if (bo->magic != TBM_BO_MAGIC)
		return 0;

	return 1;
}

static int
_tbm_bo_is_valid(tbm_bo bo)
{
	if (!bo) {
		TBM_ERR("error: bo is NULL.\n");
		return 0;
	}

	if (!_tbm_bo_magic_check(bo)) {
		TBM_ERR("error: No valid bo(%p).\n", bo);
		return 0;
	}

	return 1;
}

static void
_tbm_bo_deinit(tbm_bo bo)
{
	bo->magic = 0;

	bo->bufmgr->bo_cnt--;
	LIST_DEL(&bo->item_link);
}

tbm_bo
tbm_bo_alloc(tbm_bufmgr bufmgr, int size, int flags)
{
	tbm_bo bo;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(TBM_BUFMGR_IS_VALID(bufmgr), NULL);
	TBM_BO_RETURN_VAL_IF_FAIL(size > 0, NULL);

	bo = tbm_bufmgr_internal_alloc_bo(bufmgr, size, flags, &error);
	if (!bo) {
		/* LCOV_EXCL_START */
		TBM_ERR("tbm_bufmgr_internal_alloc_bo failed. error:%d", error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return NULL;
		/* LCOV_EXCL_STOP */
	}

	TBM_TRACE_BO("bo(%p) size(%d) refcnt(%d), flag(%s)\n", bo, size, bo->ref_cnt,
			_tbm_flag_to_str(bo->flags));

	_tbm_bufmgr_mutex_unlock();

	return bo;
}

tbm_bo
tbm_bo_ref(tbm_bo bo)
{
	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), NULL);

	bo->ref_cnt++;

	TBM_TRACE_BO("bo(%p) ref_cnt(%d)\n", bo, bo->ref_cnt);

	_tbm_bufmgr_mutex_unlock();

	return bo;
}

void
tbm_bo_unref(tbm_bo bo)
{
	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_IF_FAIL(_tbm_bo_is_valid(bo));

	TBM_TRACE_BO("bo(%p) ref_cnt(%d)\n", bo, bo->ref_cnt - 1);

	if (bo->ref_cnt <= 0) {
		_tbm_bufmgr_mutex_unlock();
		return;
	}

	bo->ref_cnt--;
	if (bo->ref_cnt == 0)
		_tbm_bo_free(bo);

	_tbm_bufmgr_mutex_unlock();
}

tbm_bo_handle
tbm_bo_map(tbm_bo bo, int device, int opt)
{
	tbm_bo_handle bo_handle;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), (tbm_bo_handle) NULL);

	if (!_tbm_bo_lock(bo, device, opt)) {
		TBM_ERR("error: fail to lock bo:%p)\n", bo);
		_tbm_bufmgr_mutex_unlock();
		return (tbm_bo_handle) NULL;
	}

	bo_handle = tbm_module_bo_map(bo->bufmgr->module, bo, bo->bo_data, device, opt, &error);
	if (bo_handle.ptr == NULL) {
		/* LCOV_EXCL_START */
		TBM_ERR("error: fail to map bo:%p error:%d\n", bo, error);
		_tbm_set_last_result(error);
		_tbm_bo_unlock(bo);
		_tbm_bufmgr_mutex_unlock();
		return (tbm_bo_handle) NULL;
		/* LCOV_EXCL_STOP */
	}

	/* increase the map_count */
	bo->map_cnt++;

	TBM_TRACE_BO("bo(%p) map_cnt(%d)\n", bo, bo->map_cnt);

	_tbm_bufmgr_mutex_unlock();

	return bo_handle;
}

int
tbm_bo_unmap(tbm_bo bo)
{
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);
	TBM_BO_RETURN_VAL_IF_FAIL(bo->map_cnt > 0, 0);

	error = tbm_module_bo_unmap(bo->bufmgr->module, bo, bo->bo_data);
	if (error != TBM_ERROR_NONE) {
		/* LCOV_EXCL_START */
		TBM_ERR("error: bo(%p) map_cnt(%d) error(%d)\n", bo, bo->map_cnt, error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return 0;
		/* LCOV_EXCL_STOP */
	}

	/* decrease the map_count */
	bo->map_cnt--;

	TBM_TRACE_BO("bo(%p) map_cnt(%d)\n", bo, bo->map_cnt);

	_tbm_bo_unlock(bo);

	_tbm_bufmgr_mutex_unlock();

	return 1;
}

tbm_bo_handle
tbm_bo_get_handle(tbm_bo bo, int device)
{
	tbm_bo_handle bo_handle;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), (tbm_bo_handle) NULL);

	bo_handle = tbm_module_bo_get_handle(bo->bufmgr->module, bo, bo->bo_data, device, &error);
	if (bo_handle.ptr == NULL) {
		/* LCOV_EXCL_START */
		TBM_ERR("error: bo(%p) bo_handle(%p) error(%d)\n", bo, bo_handle.ptr, error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return (tbm_bo_handle)NULL;
		/* LCOV_EXCL_STOP */
	}

	TBM_TRACE_BO("bo(%p) bo_handle(%p)\n", bo, bo_handle.ptr);

	_tbm_bufmgr_mutex_unlock();

	return bo_handle;
}

tbm_key
tbm_bo_export(tbm_bo bo)
{
	tbm_key ret;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	ret = tbm_module_bo_export_key(bo->bufmgr->module, bo, bo->bo_data, &error);
	if (!ret) {
		/* LCOV_EXCL_START */
		TBM_ERR("tbm_moule_bo_export_key failed. bo:%p tbm_key:%d error:%d", bo, ret, error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return ret;
		/* LCOV_EXCL_STOP */
	}

	TBM_TRACE_BO("bo:%p tbm_key:%u", bo, ret);

	_tbm_bufmgr_mutex_unlock();

	return ret;
}

tbm_fd
tbm_bo_export_fd(tbm_bo bo)
{
	tbm_fd fd;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), -1);

	fd = tbm_module_bo_export_fd(bo->bufmgr->module, bo, bo->bo_data, &error);
	if (fd < 0) {
		/* LCOV_EXCL_START */
		TBM_ERR("tbm_module_bo_export_fd filed. bo:%p tbm_fd:%d error:%d", bo, fd, error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return fd;
		/* LCOV_EXCL_STOP */
	}

	TBM_TRACE_BO("bo:%p tbm_fd:%d", bo, fd);

	_tbm_bufmgr_mutex_unlock();

	return fd;
}

tbm_bo
tbm_bo_import(tbm_bufmgr bufmgr, unsigned int key)
{
	tbm_bo bo;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(TBM_BUFMGR_IS_VALID(bufmgr), NULL);

	bo = tbm_bufmgr_internal_import_bo_with_key(bufmgr, key, &error);
	if (!bo) {
		/* LCOV_EXCL_START */
		TBM_ERR("tbm_bufmgr_internal_import_key failed. error:%d", error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return NULL;
		/* LCOV_EXCL_STOP */
	}

	TBM_TRACE_BO("import new bo(%p) ref(%d) key(%d) flag(%s) in list\n",
			  bo, bo->ref_cnt, key, _tbm_flag_to_str(bo->flags));

	_tbm_bufmgr_mutex_unlock();

	return bo;
}

tbm_bo
tbm_bo_import_fd(tbm_bufmgr bufmgr, tbm_fd fd)
{
	tbm_bo bo;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(TBM_BUFMGR_IS_VALID(bufmgr), NULL);

	bo = tbm_bufmgr_internal_import_bo_with_fd(bufmgr, fd, &error);
	if (!bo) {
		/* LCOV_EXCL_START */
		TBM_ERR("tbm_bufmgr_internal_import_fd failed. error:%d", error);
		_tbm_set_last_result(error);
		_tbm_bufmgr_mutex_unlock();
		return NULL;
		/* LCOV_EXCL_STOP */
	}

	LIST_INITHEAD(&bo->user_data_list); // TODO: remove this. build-break when it is removed.

	TBM_TRACE_BO("import bo(%p) ref(%d) fd(%d) flag(%s)\n",
			bo, bo->ref_cnt, fd, _tbm_flag_to_str(bo->flags));

	_tbm_bufmgr_mutex_unlock();

	return bo;
}

int
tbm_bo_size(tbm_bo bo)
{
	int size;
	tbm_error_e error;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	size = tbm_module_bo_get_size(bo->bufmgr->module, bo, bo->bo_data, &error);
	_tbm_set_last_result(error);

	TBM_TRACE_BO("bo(%p) size(%d)\n", bo, size);

	_tbm_bufmgr_mutex_unlock();

	return size;
}

int
tbm_bo_locked(tbm_bo bo)
{
	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	if (bo->bufmgr->bo_lock_type == TBM_BUFMGR_BO_LOCK_TYPE_NEVER) {
		TBM_ERR("bo(%p) lock_cnt(%d)\n", bo, bo->lock_cnt);
		_tbm_set_last_result(TBM_ERROR_INVALID_OPERATION);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	if (bo->lock_cnt > 0) {
		TBM_TRACE_BO("error: bo(%p) lock_cnt(%d)\n", bo, bo->lock_cnt);
		_tbm_bufmgr_mutex_unlock();
		return 1;
	}

	TBM_TRACE_BO("bo(%p) lock_cnt(%d)\n", bo, bo->lock_cnt);
	_tbm_bufmgr_mutex_unlock();

	return 0;
}

int
tbm_bo_swap(tbm_bo bo1, tbm_bo bo2)
{
	tbm_error_e error1, error2;
	int size1 = -1, size2 = -2;
	void *temp;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo1), 0);
	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo2), 0);

	TBM_TRACE_BO("before: bo1(%p) bo2(%p)\n", bo1, bo2);

	size1 = tbm_module_bo_get_size(bo1->bufmgr->module, bo1, bo1->bo_data, &error1);
	_tbm_set_last_result(error1);
	size2 = tbm_module_bo_get_size(bo2->bufmgr->module, bo2, bo2->bo_data, &error2);
	_tbm_set_last_result(error2);

	if (size1 != size2) {
		TBM_ERR("error: bo1 size(%d) and bo2 size(%d) is different.", size1, size2);
		_tbm_set_last_result(TBM_ERROR_INVALID_OPERATION);
		goto fail;
	}

	TBM_TRACE_BO("after: bo1(%p) bo2(%p)\n", bo1, bo2);

	temp = bo1->priv;
	bo1->priv = bo2->priv;
	bo2->priv = temp;

	_tbm_bufmgr_mutex_unlock();

	return 1;

fail:
	TBM_ERR("error: bo1(%p) bo2(%p)\n", bo1, bo2);
	_tbm_bufmgr_mutex_unlock();

	return 0;
}

int
tbm_bo_add_user_data(tbm_bo bo, unsigned long key,
		     tbm_data_free data_free_func)
{
	tbm_user_data *data;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	/* check if the data according to the key exist if so, return false. */
	data = user_data_lookup(&bo->user_data_list, key);
	if (data) {
		TBM_TRACE_BO("warning: user data already exist key(%ld)\n", key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	data = user_data_create(key, data_free_func);
	if (!data) {
		TBM_ERR("error: bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	TBM_TRACE_BO("bo(%p) key(%lu) data(%p)\n", bo, key, data->data);

	LIST_ADD(&data->item_link, &bo->user_data_list);

	_tbm_bufmgr_mutex_unlock();

	return 1;
}

int
tbm_bo_delete_user_data(tbm_bo bo, unsigned long key)
{
	tbm_user_data *old_data;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	if (LIST_IS_EMPTY(&bo->user_data_list)) {
		TBM_TRACE_BO("bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	old_data = user_data_lookup(&bo->user_data_list, key);
	if (!old_data) {
		TBM_TRACE_BO("bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	TBM_TRACE_BO("bo(%p) key(%lu) data(%p)\n", bo, key, old_data->data);

	user_data_delete(old_data);

	_tbm_bufmgr_mutex_unlock();

	return 1;
}

int
tbm_bo_set_user_data(tbm_bo bo, unsigned long key, void *data)
{
	tbm_user_data *old_data;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	if (LIST_IS_EMPTY(&bo->user_data_list)) {
		TBM_TRACE_BO("error: bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	old_data = user_data_lookup(&bo->user_data_list, key);
	if (!old_data) {
		TBM_TRACE_BO("error: bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	if (old_data->data && old_data->free_func)
		old_data->free_func(old_data->data);
	old_data->data = data;

	TBM_TRACE_BO("bo(%p) key(%lu) data(%p)\n", bo, key, old_data->data);

	_tbm_bufmgr_mutex_unlock();

	return 1;
}

int
tbm_bo_get_user_data(tbm_bo bo, unsigned long key, void **data)
{
	tbm_user_data *old_data;

	_tbm_bufmgr_mutex_lock();
	_tbm_set_last_result(TBM_ERROR_NONE);

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	if (!data || LIST_IS_EMPTY(&bo->user_data_list)) {
		TBM_TRACE_BO("error: bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	old_data = user_data_lookup(&bo->user_data_list, key);
	if (!old_data) {
		*data = NULL;
		TBM_TRACE_BO("error: bo(%p) key(%lu)\n", bo, key);
		_tbm_set_last_result(TBM_ERROR_INVALID_PARAMETER);
		_tbm_bufmgr_mutex_unlock();
		return 0;
	}

	*data = old_data->data;

	TBM_TRACE_BO("bo(%p) key(%lu) data(%p)\n", bo, key, old_data->data);

	_tbm_bufmgr_mutex_unlock();

	return 1;
}

int
tbm_bo_get_flags(tbm_bo bo)
{
	int flags;

	_tbm_bufmgr_mutex_lock();

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	flags = bo->flags;

	TBM_TRACE_BO("bo(%p)\n", bo);

	_tbm_bufmgr_mutex_unlock();

	return flags;
}

/* LCOV_EXCL_START */
/* internal function */
int
_tbm_bo_set_surface(tbm_bo bo, tbm_surface_h surface)
{
	_tbm_bufmgr_mutex_lock();

	TBM_BO_RETURN_VAL_IF_FAIL(_tbm_bo_is_valid(bo), 0);

	bo->surface = surface;

	_tbm_bufmgr_mutex_unlock();

	return 1;
}

void
_tbm_bo_free(tbm_bo bo)
{
	/* destory the user_data_list */
	if (!LIST_IS_EMPTY(&bo->user_data_list)) {
		tbm_user_data *old_data = NULL, *tmp;

		LIST_FOR_EACH_ENTRY_SAFE(old_data, tmp,
				&bo->user_data_list, item_link) {
			TBM_DBG("free user_data\n");
			user_data_delete(old_data);
		}
	}

	while (bo->lock_cnt > 0) {
		TBM_ERR("error lock_cnt:%d\n", bo->lock_cnt);
		_bo_unlock(bo);
		bo->lock_cnt--;
	}

	tbm_module_bo_free(bo->bufmgr->module, bo, bo->bo_data, bo->get_from_hal_surface);

	_tbm_bo_deinit(bo);

	free(bo);
}
/* LCOV_EXCL_STOP */
